<?php
/** @file
 * Infusionsoft API
 */

/**
 * Defines infusionsoft contact mappings for a module.
 *
 * To define a custom mapping it must be in the format of $map['map_name']
 * where the map name is the internal name for which the map will be called.
 *
 * Each field mapping must contain the following:
 *   'title' - What the map will be called.
 *   'description' - a short description of the map
 *   'callback' - a callback which will provide the map.
 *
 * So to add a new map it would look like:
 *   $maps['custom_map'] = array(
 *     'title' => 'My Custom Mapping',
 *     'description' => 'Send user name and email address.',
 *     'callback' => 'my_custom_function_name',
 *   );
 *
 * All the mapping logic will be contained in your mapping function.
 * The mapping function gets passed 2 parameters. first: $user, the user
 * object, and second an array of $options, which custom modules can pass
 * through. It must return mapped contact data.
 *
 * This function will look like:
 *   function my_custom_function_name($user, $options) {
 *     return array(
 *       'Username' => $user->name,
 *       'EmailAddress' => $user->mail,
 *     )
 *   }
 *
 * The key must be a field defined on the infusionsoft user table, as
 * documented here:
 * http://help.infusionsoft.com/developers/tables/user
 *
 */
function hook_infusionsoft_contact_map_info() {
  $maps['custom_map'] = array(
    'title' => 'My Custom Mapping',
    'description' => 'Send user name and email address.',
    'callback' => 'my_custom_function_name',
  );
  $maps['another_custom_map'] = array(
    'title' => 'My Other Custom Mapping',
    'description' => 'Send only the user name.',
    'callback' => 'my_custom_function_name_two',
  );

  return $maps;
}

/**
 * Alter an existing infusionsoft mapping before the mapping is used.
 */
function hook_infusionsoft_contact_map_info_alter() {

}

/**
 * Find an infusionsoft contact id by drupal username.
 *
 * @param object $user
 *   drupal user object (optional)
 *   defaults to logged in user.
 *
 * @return int|bool
 *   Returns the first contact id for the matching the email address provided by
 *   the username. If there is no username, it returns FALSE. Caches
 *   infusionsoft ids, for less hits to infusionsoft database.
 */
function infusionsoft_contactid_by_user($user = NULL) {

  return $contact_id;
}

/**
 * Find an infusionsoft contact id by email address.
 *
 * @param string $user_email
 *   Email address to look up. (optional)
 *   Defaults to the logged in user.
 *
 * @param bool $return_all
 *   If TRUE will return all contacts with an email address(optional)
 *   defaults to FALSE
 *
 * @return bool
 *   If $return_all is TRUE, returns an array of contacts matching the email
 *   address. If FALSE it will return a integer contact_id
 *
 * @link http://help.infusionsoft.com/api-docs/contactservice#findByEmail
 */
function infusionsoft_contactid_by_email($user_email = NULL, $return_all = FALSE) {

  return $return_all ? $contact_ids : $contact_id;

}

/**
 * Push a drupal user update to Infusionsoft
 *
 * NOTE: Does not update email addresses, if user is not already stored in the infusionsoft database.
 * NOTE: You cannot use this function to 'Zero out' data. For instance if you set 'Email' => '' it will not update the data.
 *
 * @param array $contact_data
 *   A keyed array which maps to Infusionsoft data tables.
 *      Example:
 *        $contact_data = array(
 *          'FirstName' => 'John',
 *          'LastName'  => 'Doe',
 *          'Email'     => 'JDoe@email.com',
 *          'Username'	=> 'johndoe'
 *        );
 * @param object $user
 *   a drupal user object.
 *
 * @param array $options
 *   an (optional) array of any of the following options:
 *     'opt_in' -  TRUE (default)
 *       Only effects new additions to infusionsoft, will opt user in to
 *        marketing emails.
 *     'opt_reason' -  'Drupal Auto Opt In' (default)
 *        Only effects new additions to infusionsoft when opt in is enabled.
 *        Will add a custom reason to why an infusionsoft user is opting in.
 *     'no_email_update' - FALSE (default)
 *       force a contact to be updated if there is no supplied email address
 *       or drupal user connection.
 *
 * @return int
 *   returns the id number of the new contact record, or the id number of
 *   the existing record.
 */
function infusionsoft_update_drupal_user($contact_data, $user, $options = array()) {


  // lets return a contact_id
  return $contact_id;
}

/**
 * Get a contact mapping.
 *
 * Lets modules add their own alternate mappings via
 * hook_infusionsoft_contact_map_info() where modules can define alternate
 * mappings.
 *
 * Provides a default contact mapping.
 *
 * @param object $user
 *   The user object to act upon.
 *
 * @param string $map_name
 *  The name of the map to send to infusionsoft.
 *
 *
 * @param array  $options
 *   An optional array of options passed through to each map callback.
 *
 * @return array
 *   Returns an array of mapped data.
 */
function infusionsoft_contact_get_map($user, $map_name = 'default', $options = array()) {

  // Returns contact data from a map.
  return $contact_data;
}

/**
 * Update an infusionsoft contact. Add new contact if none exists.
 *
 * NOTE: Does not update email addresses, if user is not already stored in the infusionsoft database.
 * NOTE: You cannot use this function to 'Zero out' data. For instance if you set 'Email' => '' it will not update the data.
 *
 * @param array $contact_data
 *   A keyed array which maps to Infusionsoft data tables.
 *   The array key needs to be the field name in which you store the value.
 *   Example:
 *     $contact_data = array(
 *       'FirstName' => 'John',
 *       'LastName'  => 'Doe',
 *       'Email'     => 'JDoe@email.com',
 *       'Username'	=> 'johndoe'
 *     );
 *
 * @param int $contact_id
 *   (optional) - if set this will update a specific contact id, this is ideal
 *   because if we don't know an infusionsoft contact id we have to lookup the id.
 *
 * @param array $options
 *   (optional) - an array of any of the following options:
 *
 *   'opt_in' -  TRUE (default)
 *     Only affects new additions to infusionsoft, will opt user in to marketing emails.
 *
 *   'opt_reason' -  'Drupal Auto Opt In' (default)
 *     Only affects new additions to infusionsoft when opt in is enabled.
 *     Will add a custom reason to why an infusionsoft user is opting in.
 *
 *   'no_email_update' - FALSE (default)
 *     Force a contact to be updated if there is no supplied email address or drupal user connection.
 *
 *   'dup_check_type' - 'Email' (default)
 *     Determines how to consider a duplicate record.
 *     Options: 'Email','EmailAndName' or 'EmailAndNameAndCompany'.
 *
 * @return int|bool
 *   Returns the id number of the new contact record, or the id number of the
 *   existing record. Returns FALSE if there is no data.
 *
 * @link http://help.infusionsoft.com/api-docs/contactservice#update
 * @link http://help.infusionsoft.com/api-docs/contactservice#addWithDupCheck
 * @link http://help.infusionsoft.com/api-docs/emailservice#optIn
 */
function infusionsoft_update_contact($contact_data, $contact_id = NULL, $options = array()) {

  return $contact_id;
}

/**
 * Send an e-mail to the given contact(s)
 *
 * @param array $contact_ids
 *   an array of (int) infusionsoft contact ids. OR a single $contact_id
 *
 * @param string $subject
 *   The email subject.
 *
 * @param string $text_body
 *   email body for text users
 *
 * @param string $html_body
 *   email body for html users
 *
 * @param string $type
 *   the type of email you are sending
 *   - options
 *     - HTML - creates an html email
 *     - Text - creates a text based email
 *     - Multipart - creates an email which is both HTML and Text
 *
 * @param string $from
 *   (optional) - the email address this template will be sent from. (defaults to owner)
 *
 * @param string $to
 *   (optional) - the email address this template will send to.
 *   This is typically the merge field "~Contact.Email~"
 *
 * @param string $cc
 *   The email address(es) this template will carbon copy. Multiples separated by comma
 *
 * @param string $bcc
 *   The email address(es) this template will blind carbon copy. Multiples separated by comma
 *
 * @return bool
 *   TRUE if email has been sent
 *
 * @link http://help.infusionsoft.com/api-docs/emailservice#sendEmail
 */
function infusionsoft_send_email($contact_ids, $subject, $text_body = '', $html_body = '', $type = 'text', $from = '~Owner.Email~', $to = '~Contact.Email~', $cc = '', $bcc = '') {

  return $email_sent ? TRUE : FALSE;
}

/**
 * Update a contact's follow up campaign.
 *
 * @param int $contact_id
 *  An infusionsoft contact id.
 *
 * @param int $followup_id
 *   An infusionsoft campaign id.
 *
 * @param $type
 *   The type of update
 *   - options
 *     - assign - assigns a contact to a follow up campaign
 *     - remove - removes a contact from a follow up campaign
 *     - pause - pauses a follow up campaign for a contact
 *     - resume - resumes a paused campaign
 *
 * @return bool
 *   TRUE if it succeeded or FALSE if it did not.
 *
 * @link http://help.infusionsoft.com/api-docs/contactservice#addToCampaign
 * @link http://help.infusionsoft.com/api-docs/contactservice#removeFromCampaign
 * @link http://help.infusionsoft.com/api-docs/contactservice#pauseCampaign
 * @link http://help.infusionsoft.com/api-docs/contactservice#resumeCampaignForContact
 */
function infusionsoft_follow_up_update($contact_id, $followup_id, $type = 'assign') {

  return $success ? TRUE : FALSE;
}

/**
 * Run an action set on a contact.
 *
 * @param int $contact_id
 *   An infusionsoft contact id.
 *
 * @param int $actionset_id
 *   An infusionsoft action set id.
 *
 * @return bool
 *   TRUE if it succeeded or FALSE if it did not.
 *
 * @link http://help.infusionsoft.com/api-docs/contactservice#runActionSequence
 */
function infusionsoft_run_actionset($contact_id, $actionset_id) {

  return $success ? TRUE : FALSE;
}

/**
 * Look up a contact group name from a group id.
 *
 * @param int $contact_group_id
 *   id of the contact group to look up the name of
 *
 * @return array
 *   array of fields
 */
function infusionsoft_get_contact_group_name($contact_group_id) {

  return $contact_group_name;
}

/**
 * Runs an infusionsoft findByField query as many times as needed to get all results.
 *
 * This is needed because Infusionsoft caps findByfield queries at 1000 responses.
 * This operation is particularly useful when doing a lookup on a foreign key, for
 * example looking up all Invoices related to a given contact.
 *
 * @param string $table
 *   The infusionsoft table to query.
 *
 * @param $field_name
 *   the data to look for
 *
 * @param $field_value
 *   the data to look for
 *
 * @param $return_fields
 *   the fields to return from the query
 *
 * @return array
 *   An array of arrays in the format:
 *   array(
 *     array(
 *       'ReturnField1' => something,
 *       'ReturnField2' => something else,
 *       ...
 *     )
 *     ...
 *   );
 *
 * NOTE: Use this very sparingly, if you have 1000s of records this can have a
 * performance impact. If you use this function frequently you will want to
 * cache it!
 *
 * @link http://help.infusionsoft.com/api-docs/dataservice#findByField
 */
function infusionsoft_findbyfield_all($table, $field_name, $field_value, $return_fields) {

  // returns an array of arrays;
  return $array;
}

/**
 * Runs an infusionsoft query as many times as needed to get all results.
 *
 * This is needed because Infusionsoft caps queries at 1000 responses.
 *
 * @param string $table
 *   The infusionsoft table to query.
 *
 * @param array $query_data
 *   The data to look for.
 *
 * @param array $selected_fields
 *   The fields to return from the query.
 *
 * @return array
 *    An array of arrays in the format:
 *      array(
 *				array(
 *					'ReturnField1' => something,
 *					'ReturnField2' => something else,
 *					...
 *				)
 *				...
 *      );
 *
 * NOTE: Use this very sparingly, if you have 1000s of records this can have a performance impact.
 *   If you use this function frequently you will want to cache it!
 *
 * @link http://help.infusionsoft.com/api-docs/dataservice#query
 */
function infusionsoft_query_all($table, $query_data, $selected_fields) {

  // returns an array of arrays;
  return $array;
}

/**
 * Primary callback function to send an API request to infusionsoft.
 *
 * Call an infusionsoft request, followed by the proper parameters.
 *
 * @param string $request_type
 *   The API function to call.
 *
 * @return bool|null|array|int|string
 *   FALSE if there is no api key
 *   OR Nothing if there is a problem with the query
 *   OR the return values for the given service and method sent via xml-rpc request.
 *
 * @link http://help.infusionsoft.com/developers/services-methods
 */
function infusionsoft_send_request($request_type = 'ContactService.load') {

  // See above, return depends on query.
  return $return;
}
